
/**
 * @typedef AwsAccessTokenType
 * @property {string} sub
 * @property {string} iss
 * @property {number} version
 * @property {string} client_id
 * @property {string} origin_jti
 * @property {string} event_id
 * @property {string} token_use - access
 * @property {string} scope - phone openid email
 * @property {number} auth_time
 * @property {number} exp
 * @property {number} iat
 * @property {number} jti
 * @property {string} username
 * @property {string[]} cognito:groups
 * @example aws.access_token
 *  {
 *   sub: 'e4886458-8061-7025-5060-5891ab32a844',
 *   iss: 'https://cognito-idp.us-east-1.amazonaws.com/us-east-1_8kErW1kco',
 *   version: 2,
 *   client_id: '6vgt83gccccb9e6ku5jla380s9',
 *   origin_jti: 'e37f9c81-7006-49f2-9973-bec85eb235e7',
 *   event_id: 'e3f0bb01-4def-481c-82ce-6cd2cb749182',
 *   token_use: 'access',
 *   scope: 'phone openid email',
 *   auth_time: 1750146979,
 *   exp: 1750147279,
 *   iat: 1750146979,
 *   jti: 'd2045ed6-ae0f-4ddf-99d5-1d7f0601a08b',
 *   username: 'e4886458-8061-7025-5060-5891ab32a844'
 *   'cognito:groups': [
 *     'cat-server-yz.rpplabs.com:admin',
 *     'localhost:admin',
 *     'dev7-ubuntu.rpplabs.com:admin'
 *   ],
 * }
 */

/**
 * @typedef CompleteTokenType
 * @description with option { complete: true }
 * @property {object} header
 * @property {string} header.kid
 * @property {string} header.alg
 * @property {AwsAccessTokenType|RppTokenType} payload
 * @property {string} signature
 */

/**
 * @typedef RppTokenType
 * @property {number} expiresIn
 * @property {object} user
 * @property {string} user.uid
 * @property {string} user.role  - [system_admin|company_admin|project_system_admin|project_operations_admin|project_maintainer|project_user|super_user|system_admin]
 * @property {string} user.refresh_token
 * @property {string} user.company_uid
 * @property {string} user.status - [ACTIVE|INACTIVE]
 * @property {number} iat
 * @example
 *  {
 *   expiresIn: 1800,
 *   user: {
 *     uid: 'L2qskpcESZWbSGV7-Oku9A',
 *     role: 'system_admin',
 *     email: 'lab+8031',
 *     refresh_token: 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHBpcmVzSW4iOjEwMzY4MDAwLCJpZCI6Ink5N0tpVVFFUS1fWWVBM29vWGVWY2ciLCJpYXQiOjE3NDkxMDczNTh9.Fv5Fod1wl3MvYZ8iJrC9h5Zj4Z9HdB6jvCLk879ZtgE',
 *     company_uid: 'AA_Dn4M4Tg64gqE6N91RO1',
 *     status: 'ACTIVE'
 *   },
 *   iat: 1750165628
 * }
 */
