const path = require('path');
const gateway = require('express-gateway');
const fs = require('fs');

require(`dotenv-defaults`).config();

function readSecret(keyFile, defaultKeyFile) {
    function readFile(file) {
        try {
            return fs.readFileSync(file, 'utf8');
        } catch (e) {
            console.error(`[api-gateway] [error]: Fails to read key file "${file}"`);
            console.error(e);
            process.exit(-1);
        }
    }
    
    if (process.env.NODE_ENV !== 'prod') {
        // must be same as rtls-server/server/node_modules/enums/common.js CONF_DEFAULT.AUTH_TOKEN_SECRET
        return process.env.RPP_AUTH_TOKEN_SECRET_DEV;
    }
    if (fs.existsSync(defaultKeyFile)) {
        return readFile(defaultKeyFile);
    } else if(fs.existsSync(keyFile)) {
        return readFile(keyFile);
    } else {
        console.error(`[api-gateway] [error]: Key file "${keyFile}" not found`);
        process.exit(-1);
    }
    return false;
}
process.env.RPP_GW_AUTH_TOKEN_SECRET = readSecret(process.env.RPP_AUTH_SECRET_FILE, process.env.RPP_AUTH_SECRET_FILE_DEFAULT);

gateway()
  .load(path.join(__dirname, 'config'))
  .run();
